/* Shared library add-on to iptables for the ANTISPOOF target
 * (C) 2000 by Harald Welte <laforge@gnumonks.org>
 *
 * $Id: libipt_ANTISPOOF.c,v 1.3 2006/02/25 16:12:03 nprasad Exp $
 *
 * This program is distributed under the terms of GNU GPL
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <xtables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_ANTISPOOF.h>

static void ANTISPOOF_help(void)
{
	printf("ANTISPOOF target Command is of the form\n"
	       "iptables -t filter -I INPUT  -j ANTISPOOF --log-enable 1 or 0\n");
}

static void
parse_antispoof(const char *s, struct ipt_antispoof_target_info *info)
{
	unsigned int value = 0;

	xtables_strtoui(s, NULL, &value, 0, 0);
	info->logStatus = value;
}

static int ANTISPOOF_parse(int c, char **argv, int invert, unsigned int *flags,
		 const void *entry, struct xt_entry_target **target)
{
	struct ipt_antispoof_target_info *antispoofinfo
	    = (struct ipt_antispoof_target_info *)(*target)->data;

	switch (c) {
	case '1':
		if (*flags)
			xtables_error(PARAMETER_PROBLEM,
				      "ANTISPOOF: Cant specify --log-enable twice\n");

		parse_antispoof(optarg, antispoofinfo);
		*flags = 1;
		break;

	default:
		return 0;
	}

	return 1;

}

static void ANTISPOOF_check(unsigned int flags)
{
#ifdef DEBUG
	if (!flags)
		fprintf(stderr,
			"ANTI SPOOF : Parameter --log-enable 0/1 is required "
			"has parameters.\n");
#endif

}

static void ANTISPOOF_save(const void *ip, const struct xt_entry_target *target)
{
	const struct ipt_antispoof_target_info *antispoofinfo =
	    (const struct ipt_antispoof_target_info *)target->data;
	printf("--log-enable %d ", antispoofinfo->logStatus);
}

static void ANTISPOOF_print(const void *ip,
		  const struct xt_entry_target *target, int numeric)
{
	const struct ipt_antispoof_target_info *antispoofinfo =
	    (const struct ipt_antispoof_target_info *)target->data;
	printf("log-enable ");
	printf("%d ", antispoofinfo->logStatus);

}

static struct option ANTISPOOF_opts[] = {
	{"log-enable", 1, 0, '1'},
	{0}
};

static struct xtables_target antispoof_tg_reg = {
	.next			= NULL,
	.name			= "ANTISPOOF",
	.version		= XTABLES_VERSION,
	.family			= NFPROTO_IPV4,
	.size			= XT_ALIGN(sizeof(struct ipt_antispoof_target_info)),
	.userspacesize	= XT_ALIGN(sizeof(struct ipt_antispoof_target_info)),
	.help			= ANTISPOOF_help,
	.parse			= ANTISPOOF_parse,
	.final_check	= ANTISPOOF_check,
	.print			= ANTISPOOF_print,
	.save			= ANTISPOOF_save,
	.extra_opts		= ANTISPOOF_opts
};

void _init(void)
{
	xtables_register_target(&antispoof_tg_reg);
}
