--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Feb 06, 2008

#### File: ddns.lua
#### Description: DDNS functions

#### Revisions: 
01e 13Mar13, Smita P. Added ddns.deleteVlanDdns function to delete DNS service added for VLAN enabled WANs.
01d 10Feb13, Smita P. Fixed spr 36697.
01c 18Aug11, Smita P. Modified ddns.configWrap function to support 4 WANS in SRX5308
01b, 15Jul08, shv Added Wrapper function to support WAN1 and WAN2 DDNS.
]]--


--************* Requires *************

--************* Initial Code *************
--package ddns
ddns = {}
--************* Functions *************
-- ddns update Configuration
function ddns.updateConfig (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return false
        elseif (operation == "edit") then
            return db.update(DBTable,inputTable,rowid)
        elseif (operation == "delete") then
            return false
        end
    end
end

-- ddns config
function ddns.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    valid = ddns.updateConfig (inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DDNS_CONFIG_FAILED"
    end
end

-- Gui Wrapper function to configure WAN1, WAN2, WAN3 and WAN4 DDNS.
function ddns.configGuiWrap (inputTable, noOfWans, ddnsServiceType)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
	for i = 1, tonumber(noOfWans) do
		local ddnsTable = {}
		ddnsTable["ddns.ddnsService"] = inputTable["ddns.ddnsService" .. i] or '0'
		if (ddnsTable["ddns.ddnsService"] ~= "0") then
            ddnsTable["ddns.hostname"] = inputTable["ddns.hostname" .. i]
            ddnsTable["ddns.username"] = inputTable["ddns.username" .. i]
            ddnsTable["ddns.password"] = inputTable["ddns.password" .. i]
            ddnsTable["ddns.wildflag"] = inputTable["ddns.wildflag" .. i]
            ddnsTable["ddns.timePeriod"] = inputTable["ddns.timePeriod" .. i]
        end
        local oldDdnsServiceType = db.getAttribute ("ddns", "_ROWID_", i, "ddnsService") or '0'
        if ((ddnsServiceType == ddnsTable["ddns.ddnsService"]) or ((oldDdnsServiceType == ddnsServiceType) and (ddnsTable["ddns.ddnsService"] == '0'))) then
            valid = ddns.updateConfig (ddnsTable, i, "edit")
        else
            valid = true
        end
	end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DDNS_CONFIG_FAILED"
    end
end

-- Wrapper function to configure WAN1, WAN2, WAN3 and WAN4 DDNS.
function ddns.configWrap (inputTable, noOfWans)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
	for i = 1, tonumber(noOfWans) do
		local ddnsTable = {}
		ddnsTable["ddns.ddnsService"] = inputTable["ddns.ddnsService" .. i] or '0'
		if (ddnsTable["ddns.ddnsService"] ~= "0") then
            ddnsTable["ddns.hostname"] = inputTable["ddns.hostname" .. i]
            ddnsTable["ddns.username"] = inputTable["ddns.username" .. i]
            ddnsTable["ddns.password"] = inputTable["ddns.password" .. i]
            ddnsTable["ddns.wildflag"] = inputTable["ddns.wildflag" .. i]
            ddnsTable["ddns.timePeriod"] = inputTable["ddns.timePeriod" .. i]
        end
        valid = ddns.updateConfig (ddnsTable, i, "edit")

        if (not valid) then
        	break
        end
	end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DDNS_CONFIG_FAILED"
    end
end

-- function to get ddns information.
function ddns.configGet (intName)
	return db.getRow ("ddns", "interfaceName", intName)
end

function ddns.deleteVlanDdns (rowIds)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transactions
    local valid = true
    for k, v in pairs (rowIds) do
        local inTable = {}
        inTable["ddns._ROWID_"] = v;
        inTable = db.getRow ("ddns", "ddns._ROWID_", v)
        if (db.existsRow("ddns", "_ROWID_", v)) then
            inTable["ddns.ddnsService"] = "0"
            valid = ddns.updateConfig (inTable, v, "edit")
        end
        if (valid == false) then
            break
        end
    end
    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DDNS_CONFIG_FAILED"
    end
end
