--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: ifDev.lua
#### Description: ifDev functions

#### Revisions:
01a, 05mar13, added api for cms support
None
]]--


--************* Requires *************
require "ifDevLib"

--************* Initial Code *************

--package ifDev
ifDev = {}

--table packages
networkInterface = {}

--************* Functions *************

-- networkInterface config
function networkInterface.config (inputTable, rowid, operation)
	if(ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED","ADMIN_REQD"	
	end

	-- validate
	if (networkInterface.inputvalidate(inputTable, operation)) then
		if (operation == "add") then
			return nil
		elseif (operation == "edit") then
			return db.update("networkInterface", inputTable, rowid)
		elseif (operation == "delete") then
			return nil
		end
	end
	return false
end

-- networkInterface inputvalidate
function networkInterface.inputvalidate (inputTable, operation)
	if (true) then
		return db.typeAndRangeValidate(inputTable)
	end
	return false
end

-- networkInterface config
function networkInterface.autoRefreshSet (inputTable, rowid, operation)
	if(ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED","ADMIN_REQD"	
	end

    db.beginTransaction() --begin transaction
    local valid = false

	-- validate
	if (networkInterface.inputvalidate(inputTable, operation)) then
		if (operation == "add") then
			valid = db.insert("interfaceStatsParams", inputTable, rowid)
		elseif (operation == "edit") then
			valid = db.update("interfaceStatsParams", inputTable, rowid)
		elseif (operation == "delete") then
			valid = false
		end
	end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "AUTOREFRESH_CONFIG_FAILED"
    end
end

function networkInterface.configUsageSnaps (inputTable, operation)
    if (networkInterface.inputvalidate(inputTable, operation)) then
        if (operation == "delete") then
            valid = db.execute ("DELETE FROM ifUsageReportConfig")
        elseif (operation == "add") then
            valid = db.execute ("INSERT INTO ifUsageReportConfig VALUES (" .. inputTable["ifUsageReportConfig.maxSnaps"] .. ", 0)")
        else
            valid = db.execute ("UPDATE ifUsageReportConfig SET maxSnaps=" .. inputTable["ifUsageReportConfig.maxSnaps"])
        end

        if (valid) then
            return "OK", "STATUS_OK"
        end
    end

    return "ERROR", "IFDEV_USAGE_CONFIG_FAILED"
end


function networkInterface.ifaceUsageReport (fileName)
    local repFile = assert (io.open (fileName, "w"))
    local statsTable = db.getTable ("interfaceUsage", nil, "SELECT * FROM interfaceUsage ORDER BY entryNum")
    local usageInfoTbl = db.getTable ("interfaceUsage", nil, "SELECT count(*)/max(entryNum) as numRecsPerGroup FROM interfaceUsage")
	local rowIdx = 0
    local nRecsPerGrp = usageInfoTbl[1]["interfaceUsage.numRecsPerGroup"];
    local csvBuff = ""
    util.appendDebugOut ("------------------- CSV BUFFER -------------------------<br>")

	for col,val in pairs(statsTable) do
		rowIdx = rowIdx + 1
        local row = statsTable[rowIdx]
        csvBuff = ""
        if (((rowIdx - 1) % nRecsPerGrp) == 0) then
            if ((rowIdx - 1) ~= 0) then
                csvBuff = "\n\n"
            end
            csvBuff = csvBuff .. "Timestamp," .. row["interfaceUsage.entryTime"]
            util.appendDebugOut (csvBuff .. "<br>")
            repFile:write (csvBuff)
            csvBuff = "\nInterface Name, " ..
                      "RX Packets, TX Packets,  " ..
                      "RX Bytes, RX Byte Rate (bytes/sec), " ..
                      "TX Bytes, TX Byte Rate (bytes/sec), " ..
                      "RX Errors, RX Error Rate (errors/sec), " ..
                      "TX Errors, TX Error Rate (errors/sec)"
            util.appendDebugOut (csvBuff .. "<br>")
            repFile:write (csvBuff)
         end
        
        csvBuff = "\n\"" .. row["interfaceUsage.interfaceName"] .. "\"," ..
                  "\"" .. row["interfaceUsage.rx_packets"] .. "\"," ..
                  "\"" .. row["interfaceUsage.tx_packets"] .. "\"," ..
                  "\"" .. row["interfaceUsage.rx_bytes"] .. "\"," ..
                  "\"" .. row["interfaceUsage.rx_bytes_rate"] .. "\"," ..
                  "\"" .. row["interfaceUsage.tx_bytes"] .. "\"," ..
                  "\"" .. row["interfaceUsage.tx_bytes_rate"] .. "\"," ..
                  "\"" .. row["interfaceUsage.rx_errors"] .. "\"," ..
                  "\"" .. row["interfaceUsage.rx_error_rate"] .. "\"," ..
                  "\"" .. row["interfaceUsage.tx_errors"] .. "\"," ..
                  "\"" .. row["interfaceUsage.tx_error_rate"] .. "\"";
        util.appendDebugOut (csvBuff .. "<br>")
        repFile:write (csvBuff)
    end
    repFile:write ("\n")
    repFile:close ()
    util.appendDebugOut ("--------------------------------------------------------<br>")  
end

--[[
--*****************************************************************************
-- ifDev.statsRefresh - refresh interface statistics 
-- 
-- This function refreshes interface statistics.
--
-- RETURN: number of instances
]]--

function ifDev.statsRefresh ()
    os.execute("ifDevUpdateStats /tmp/system.db")
end


function ifDev.ipSubnet(ipAddr,subnetMask,startAddr,endAddr)

    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "Administrator privileges are required"
    end

    local ipAddr = ipAddr
    local subnetMask = subnetMask
    local startAddr = startAddr
    local endAddr = endAddr
    local status = ifDevLib.ipSubnet(ipAddr,subnetMask,startAddr,endAddr)
    
    return status

end

--[[
--*******************************************************************************
]]--

function ifDev.ipAddrRange(startAddr,endAddr)

    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "Administrator privileges are required"
    end

    local startAddr = startAddr
    local endAddr = endAddr
    local range = ifDevLib.ipRange(startAddr,endAddr)
    
    return range

end

--checking the same subnet ip is available in ipaddress table. if so return INTERFACE name and ERROR
function ifDev.IPv6SubnetAlreadyInUse(inputIfName, ipv6Addr, subnetLength)
	local count = 0
	if(inputIfName ~= nil and ipv6Addr ~= nil and subnetLength ~= nil) then
		local qryRow_ipAddressTable = db.getRowsWhere("ipAddressTable","addressFamily = 10 AND LogicalIfName != '" ..inputIfName .. "'" ) or {}
		if(#qryRow_ipAddressTable ~= 0)then
			for k,v in pairs (qryRow_ipAddressTable) do
				count = count + 1
				local row = qryRow_ipAddressTable[count]
				if(row ~= nil) then
					if(row["ipAddressTable.ipAddress"] ~= nil and row["ipAddressTable.ipv6PrefixLen"]  ~= nil) then
						if(ifDevLib.ifDevIPv6SubnetAddress(ipv6Addr,subnetLength) == ifDevLib.ifDevIPv6SubnetAddress(row["ipAddressTable.ipAddress"],row["ipAddressTable.ipv6PrefixLen"]))then
							return "ERROR" 
						end
					end
				end
			end
		end
	end
	return "OK"
end

function ifDev.twoIPv6AddressInSameSubnet(ipv6Addr1, ipv6Addr2,subnetLength)
	if(ipv6Addr1 ~= nil and ipv6Addr2 ~= nil) then
		if(ifDevLib.ifDevIPv6SubnetAddress(ipv6Addr1,subnetLength) ~= ifDevLib.ifDevIPv6SubnetAddress(ipv6Addr2,subnetLength))then
			return "ERROR" 
		end
	end
	return "OK"
end

function ifDev.interfaceOidNameGet(logicalIfName)
    row = db.getRow("Zones", "ZoneLogicalIfName", logicalIfName)
    if (row ~= nil) then
        if (row["Zones.ZoneType"] == "INSECURE") then
            return "system:nwConfig:wans:wan", row["Zones.xmlId"]
        elseif (row["Zones.ZoneType"] == "SECURE") then
            local vlanId = "1"
            if (string.len (logicalIfName) > 3) then
                vlanId = string.sub(logicalIfName, 4)
            end
            return "system:nwConfig:lan:lanSetup:vlanProfiles:vlanProfile", vlanId
        elseif (row["Zones.ZoneType"] == "PUBLIC") then
            return "system:nwConfig:dmzSetup", row["Zones.xmlId"]
        else
            return "ERROR"
        end
    end
	return "ERROR"
end
