 -- igmpCLI.lua
-- 
-- TeamF1
-- www.TeamF1.com
--
--
-- Description
-- CLISH IGMP LUA ROUTINES
--
-- load required library
require "teamf1lualib/igmp"

function igmpPassthroughConfigInit(args)
    local configRow = db.getRow("Igmp","Igmp._ROWID_",1)
    --print("rowid ="..configRow["Igmp._ROWID_"])
    --print("enable ="..configRow["Igmp.IgmpEnable"])
    --print("upiface ="..configRow["Igmp.upstreamInterface"])
    return 1, configRow
end

function igmpPassthroughConfigSave(configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = ""
    local statusMessage = ""
    DBTable = "Igmp"

     errorFlag, statusCode = igmp.config (configRow, configRow["Igmp._ROWID_"], "edit")
     --print("igmp.config -bd -saving")
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)    

    return errorFlag, statusMessage
end

function igmpPassthroughConfigInputVal(configRow)
    local tmpRow = db.getRow("WanMode","WanMode._ROWID_",1)
    local tmpRow2 = db.getRow("Igmp","Igmp._ROWID_",1)
    if(tmpRow == nil or tmpRow2 == nil) then
        printCLIError("query(s) failed")
        return false
    end
    if(tmpRow["WanMode.Wanmode"] ~= "1") then
        if(configRow["Igmp.upstreamInterface"] ~= tmpRow2["Igmp.upstreamInterface"]) then
            printCLIError("Upstream Interface is configurable only in load balancing mode")
            return false
        end
    end
    if(configRow["Igmp.IgmpEnable"] ~= "1") then
        if(configRow["Igmp.upstreamInterface"] ~= tmpRow2["Igmp.upstreamInterface"]) then
            printCLIError("Upstream Interface is configurable only when Igmp passthrough is enabled")
            return false
        end
    end

    return true

end

function igmpPassthroughConfigInputVal_fvs318n(configRow)
    local tmpRow = db.getRow("WanMode","WanMode._ROWID_",1)
    local tmpRow2 = db.getRow("Igmp","Igmp._ROWID_",1)
    if(tmpRow == nil or tmpRow2 == nil) then
        printCLIError("query(s) failed")
        return false
    end
    
    return true

end


function igmpAllowedNetConfigInit(args)
    rowId = args[1]
    if(rowId == nil) then
        configRow ={}
        configRow["allowedNets._ROWID_"] = "-1"
    else
        configRow = {}
        configRow["allowedNets._ROWID_"] = rowId
        configRow = db.getRow("allowedNets", "_ROWID_", rowId)
    end
    
    return configRow["allowedNets._ROWID_"], configRow
end

function igmpAllowedNetConfigSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "allowedNets"

    errorFlag, statusCode = igmp.allowedNetworkConfig(configRow, "-1", "add")
    if (errorFlag == "OK") then
        db.save()
        --print ("db.save called")
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage

end

function igmpAllowedNetConfigInputVal(configRow)
    if(configRow["allowedNets.networkAddr"] == "" or configRow["allowedNets.networkAddr"] == nil) then
        printCLIError("please give a network address")
        return false
    end
    if(configRow["allowedNets.NetMask"] == "" or configRow["allowedNets.NetMask"] == nil) then
        printCLIError("please give a network Mask")
        return false
    end
    return true
end

function igmpAlternateNetConfigDel(args)
    DBTable = "allowedNets"
    local errorFlag = "ERROR" -- default 
    local statusCode = "IGMP_CONFIG_FAILED"
    local statusMessage = ""
    local row = args[1]
    local rows = {}
    local configRow ={}
    local configRow = db.getRow("allowedNets","_ROWID_",row)
    if(configRow == nil) then
        printCLIError("This row doesn't exists")
        return false
    else
        table.insert (rows, configRow["allowedNets._ROWID_"])
        --print("row num to delete:"..configRow["allowedNets._ROWID_"])
        if (rows) then
            errorFlag, statusCode = igmp.deleteAllowedNetwork(rows)
        end
        -- save db if no error
        if (errorFlag == "OK") then db.save() end
        statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
        return errorFlag, statusMessage
    end
end

function getIgmpProxyConfig()
    print ("IGMP System configuration")
    row = db.getRow("Igmp","Igmp._ROWID_",1)
    if(row["Igmp.IgmpEnable"] == "0") then
        print("IGMP Pass Through : Disabled")
    else
        print("IGMP Pass Through : Enabled")
    end
    print("Up Stream Interface :"..row["Igmp.upstreamInterface"])    
end

function getIgmpAllowedNetworks()
    local resultTab = {}
    local allowedNetsTab = db.getTable("allowedNets")
	local idx = 0
	for k,v in pairs(allowedNetsTab) do
	    idx = idx + 1
	    local row = allowedNetsTab[idx]
        resTab.insertField (resultTab,"Row Id",row["allowedNets._ROWID_"])
        resTab.insertField (resultTab, "Network Address", row["allowedNets.networkAddr"])
        resTab.insertField (resultTab, "Subnet Mask", row["allowedNets.NetMask"])
    end
    printLabel ("Allowed Networks")
    resTab.print (resultTab, 0)
end
