--[[
#### Hussain Vali
#### TeamF1
#### www.TeamF1.com
#### Mar 28, 2008

#### File: lanGroups.lua
#### Description: Lan Groups functions

#### Revisions:
None
-- Modification History
30Jan14,akr Fixed SPR 41795
]]--


--************* Requires *************
require "langroupsLib"
require "ifDevLib"
require "teamf1lualib/ifDev"
--************* Initial Code *************

lanGroups = {}
lanHost = {}

--************* Functions *************

-- lan Groups config
function lanGroups.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

	-- validate
	if (lanGroups.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            if (checkValidGroupName (inputTable["LanGroup.GroupName"])) then
                valid = db.insert("LanGroup", inputTable)
            else
                db.rollback()
                return "ERROR", "GROUPS_NAME_ALREADY_EXIXSTS"
            end
        elseif (operation == "edit") then
            valid = false
        elseif (operation == "delete") then
            for k,v in pairs(inputTable) do
                local groupName = db.getAttribute("LanGroup", "LanGroup._ROWID_", v, "GroupName")
                local inUseRows = db.existsRowWhere("BlockSites", "KeywordBlockingTo = '" .. groupName .. "'")
                if (inUseRows) then
                    db.rollback()
		            return "ERROR", "GROUPNAME_BLOCKED_KEYWORD_MAP_IN_USE"
                end
                local usingRows = db.getRowsWhere("LanHost", "GroupName = '" .. groupName .. "'")
                if (usingRows ~= nil) then
                    for l,m in pairs(usingRows) do
                        valid = lanHost.config (m, "-1", "delete")
                        if (valid) then
                        else
                            db.rollback()
    		                return "ERROR", "LAN_HOST_CONFIG_FAILED"
                        end
                    end
                end
            end
            valid = db.delete("LanGroup", inputTable)
        end
	end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LAN_GROUPS_CONFIG_FAILED"
    end
end

-- lan Groups inputvalidate
function lanGroups.inputvalidate (inputTable, operation)
	if (true) then
		return db.typeAndRangeValidate(inputTable)
	end
	return false
end

function checkValidGroupName (grpName)
	local table = db.getTable ("LanGroup")
	local i = 0
	for k,v in pairs(table) do
		i = i + 1
		local row = table[i]
        if (string.lower (grpName) == string.lower (row["LanGroup.GroupName"])) then
            return false
        end
    end
    return true
end

function lanHost.config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	db.beginTransaction() --begin transaction
	local valid = false
    
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert("LanHost", inputTable)
        elseif (operation == "edit") then
            valid = db.update("LanHost", inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete("LanHost", inputTable)
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "LAN_HOST_CONFIG_FAILED"
	end
end

--[[
/******************************************************************************
*
* lanGroups.editGroupName - Edits Group Name
*
* RETURNS: "OK" or 'ERROR'
*/
]]--

function lanGroups.editGroupName (inputTable)
    -- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	-- Declaring flag
	
	local valid = false
	
    -- get the row from LanGroup table using OID value
    DBTable = "LanGroup"
    if(inputTable["LanGroup.OID"] == nil) then
        return "ERROR", "IN_VALID_GROUP_NAME"
    end
    local lanGroupRow = db.getRow (DBTable, "LanGroup._ROWID_", inputTable["LanGroup.OID"])
    
    -- update lanGroupRow with the received Group Name
    if(inputTable["LanGroup.GroupName"]) then
        lanGroupRow["LanGroup.OID"] = inputTable["LanGroup.OID"]
        lanGroupRow["LanGroup.GroupName"] = inputTable["LanGroup.GroupName"]
    end
    
    -- Check if the received GroupName already exists in table
    -- If it exists then return message.
    
    if(db.existsRowWhere ("LanGroup", "GroupName='" .. inputTable["LanGroup.GroupName"] .. "' AND _ROWID_ != '" .. lanGroupRow["LanGroup._ROWID_"] .. "'"))  then
        db.rollback ()
        return "ERROR", "GROUPS_NAME_ALREADY_EXIXSTS"
    end
    
    -- Begin Transaction
        
    db.beginTransaction()
    
    valid = db.update (DBTable, lanGroupRow, lanGroupRow["LanGroup._ROWID_"])
    
    if(valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback ()
        return "ERROR", "EDIT_GROUP_CONFIG_FAILED"
    end      
end

--[[
/******************************************************************************
*
* lanGroups.addLanMultiHoming - Adds Secondary Lan IPs
*
* RETURNS: "OK" or 'ERROR'
*/
]]--
function lanGroups.addLanMultiHoming (inputTable)
    -- if not allowed to edit/add
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	-- Declaring flags
	local valid = false

    if (DBTable == "ifStaticAliasAddrTbl") then
        if( ifDev.IPv6SubnetAlreadyInUse("dummyLAN",inputTable["ifStaticAliasAddrTbl.StaticIp"] , inputTable["ifStaticAliasAddrTbl.PrefixLength"] )== "ERROR")then
            print("IPv6 address already in use, try other ip.")
            return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
        end
    end

    if (DBTable == "ifStaticIPAlias") then
    local ifstaticRows = db.getRowsWhere("ifStatic","AddressFamily = 2")
    local j = 0
    if (#ifstaticRows ~= 0) then
        for k,v in pairs(ifstaticRows) do
            j = j + 1
            local ifstatic = ifstaticRows[j]
            if (string.find (ifstatic["ifStatic.LogicalIfName"],"WAN")) then
                local ipAddrRow = db.getRowWhere("ipAddressTable","addressFamily = 2 and LogicalIfName='".. ifstatic["ifStatic.LogicalIfName"] .."'")
                if (ipAddrRow ~= nil) then
                    ifstatic["ifStatic.StaticIp"] = ipAddrRow["ipAddressTable.ipAddress"] or ""
                    ifstatic["ifStatic.NetMask"] = ipAddrRow["ipAddressTable.subnetMask"] or ""
                end
            end
            if (ifstatic ~= nil and ifstatic["ifStatic.StaticIp"] ~= nil and
                ifstatic["ifStatic.StaticIp"] ~= "0.0.0.0" and ifstatic["ifStatic.StaticIp"] ~= "") then
				errorFlag = ifDevLib.ipSubnet(ifstatic["ifStatic.StaticIp"],ifstatic["ifStatic.NetMask"],inputTable["ifStaticIPAlias.IPAddress"])
				local var ="Error: Invalid Start IP Address. " .. inputTable["ifStaticIPAlias.IPAddress"] .." is out of subnet range"
				if(errorFlag ~= var) then
            	    return "ERROR", "IP subnet conflicting with "..ifstatic["ifStatic.LogicalIfName"]
                end
            end
        end
    end
    end

    if (DBTable == "ifStaticIPAlias") then
        maxId = db.columnMaxValueWhere("ifStaticIPAlias", "xmlId", "LogicalIfName = 'LAN'")
        if(maxId == "" or maxId == nil) then
            maxId = -1
        end
        inputTable["ifStaticIPAlias.xmlId"] = tonumber(maxId) + 1
    elseif (DBTable == "ifStaticAliasAddrTbl") then
        maxId = db.columnMaxValueWhere("ifStaticAliasAddrTbl", "xmlId", "LogicalIfName = 'LAN'")
        if(maxId == "" or maxId == nil) then
            maxId = -1
        end
        inputTable["ifStaticAliasAddrTbl.xmlId"] = tonumber(maxId) + 1
    end
	-- Begin Transaction
		
	db.beginTransaction()
	
	if(db.typeAndRangeValidate(inputTable)) then
	    valid = db.insert (DBTable, inputTable)
	end	
	if(valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback ()
        return "ERROR", "MULTI_LAN_CONFIG_FAILED"
    end
	
end

--[[
/******************************************************************************
*
* lanGroups.deleteLanMultiHoming - Deletes Secondary Lan IPs
*
* RETURNS: "OK" or 'ERROR'
*/
]]--
function lanGroups.deleteLanMultiHoming (inputTable)
    -- if not allowed to edit/add
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	local valid = false -- flag
	
	-- Begin Transaction
			
	db.beginTransaction()	
	for k,v in pairs (inputTable) do		
	   valid = db.deleteRow (DBTable,"_ROWID_",v)
	    if(valid == false) then
	        break
	    end
	end
	
	if(valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback ()
        return "ERROR", "MULTI_LAN_DELETE_FAILED"
    end	
end

--[[
/******************************************************************************
*
* lanGroups.editLanMultiHoming - edits Secondary Lan IPs
*
* RETURNS: "OK" or 'ERROR'
*/
]]--

function lanGroups.editLanMultiHoming (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    local valid = false

    if (DBTable == "ifStaticAliasAddrTbl") then
        local ipRowId = ""
        local oldifStaticAliasAddrTblRow = db.getRow("ifStaticAliasAddrTbl", "_ROWID_", rowid)
		if(oldifStaticAliasAddrTblRow == nil or oldifStaticAliasAddrTblRow == {} ) then
			return "ERROR", "ERROR"
		end
        local prevStaticIp = string.lower(oldifStaticAliasAddrTblRow["ifStaticAliasAddrTbl.StaticIp"])
        ipRowId = db.getAttribute("ipAddressTable","ipAddress",prevStaticIp ,"_ROWID_")
		if(inputTable["ifStaticAliasAddrTbl.PrefixLength"] ~= oldifStaticAliasAddrTblRow["ifStaticAliasAddrTbl.PrefixLength"]) then
        	if( ifDev.IPv6SubnetAlreadyInUse("dummyLAN",inputTable["ifStaticAliasAddrTbl.StaticIp"] , inputTable["ifStaticAliasAddrTbl.PrefixLength"], ipRowId )== "ERROR")then
            	print("IPv6 address already in use, try other ip.")
            	return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
        	end
		elseif (inputTable["ifStaticAliasAddrTbl.PrefixLength"] == oldifStaticAliasAddrTblRow["ifStaticAliasAddrTbl.PrefixLength"] ) then
			if(ifDev.twoIPv6AddressInSameSubnet(prevStaticIp, inputTable["ifStaticAliasAddrTbl.StaticIp"],inputTable["ifStaticAliasAddrTbl.PrefixLength"]) == "ERROR") then
				if( ifDev.IPv6SubnetAlreadyInUse("dummyLAN",inputTable["ifStaticAliasAddrTbl.StaticIp"] , inputTable["ifStaticAliasAddrTbl.PrefixLength"], ipRowId )== "ERROR")then
					print("IPv6 address already in use, try other ip.")
					return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
				end
			end
		end
    end

    if (DBTable == "ifStaticIPAlias") then
        local ifstaticRows = db.getRowsWhere("ifStatic","AddressFamily = 2")
        local j = 0
        if (#ifstaticRows ~= 0) then
            for k,v in pairs(ifstaticRows) do
                j = j + 1
                local ifstatic = ifstaticRows[j]
                if (string.find (ifstatic["ifStatic.LogicalIfName"],"WAN")) then
                    local ipAddrRow = db.getRowWhere("ipAddressTable","addressFamily = 2 and LogicalIfName='".. ifstatic["ifStatic.LogicalIfName"] .."'")
                    if (ipAddrRow ~= nil) then
                        ifstatic["ifStatic.StaticIp"] = ipAddrRow["ipAddressTable.ipAddress"] or ""
                        ifstatic["ifStatic.NetMask"] = ipAddrRow["ipAddressTable.subnetMask"] or ""
                    end
                end
                if (ifstatic ~= nil and ifstatic["ifStatic.StaticIp"] ~= nil and
                    ifstatic["ifStatic.StaticIp"] ~= "0.0.0.0" and ifstatic["ifStatic.StaticIp"] ~= "") then
                    errorFlag = ifDevLib.ipSubnet(ifstatic["ifStatic.StaticIp"],ifstatic["ifStatic.NetMask"],inputTable["ifStaticIPAlias.IPAddress"])
                    local var ="Error: Invalid Start IP Address. " .. inputTable["ifStaticIPAlias.IPAddress"] .." is out of subnet range"
                    if(errorFlag ~= var) then
                        return "ERROR", "IP subnet conflicting with "..ifstatic["ifStatic.LogicalIfName"]
                    end
                end
            end
        end
    end

    db.beginTransaction()
    if(db.typeAndRangeValidate(inputTable)) then
        valid = db.update (DBTable, inputTable, rowid)
    end

    if(valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback ()
        return "ERROR", "MULTI_LAN_CONFIG_FAILED"
    end
end


function lanGroups.vlanProf(hostipAddr)

    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        NextPage = ""
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local hostipAddr = hostipAddr
    local status = langroupsLib.GetVlanProfName(hostipAddr)
    
    return status

end
