-- radiusCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 10may13,nik fixed SPR 36703
-- 28jan08,gnm modified error message text in radiusCfgInputVal() 
-- 15nov07 gnm written
--
-- Description
-- CLI Radius routines

require "teamf1lualib/radius"

--
--
function radiusCfgInputVal (configRow)
    if (configRow["radiusClient.isEnable"] == "1") then
        if (configRow["radiusClient.authsecret"] == nil or configRow["radiusClient.authsecret"] == '') then
            printCLIError ("Invalid RADIUS secret.")
	        return false
        end
        if (configRow["radiusClient.authserver"] == nil or configRow["radiusClient.authserver"] == '' or
            configRow["radiusClient.authserver"] == "0.0.0.0") then
            printCLIError ("Invalid RADIUS server IP.")
	        return false
        end
        if (configRow["radiusClient.nasIdentifier"] == nil or configRow["radiusClient.nasIdentifier"] == '') then
            printCLIError ("Invalid RADIUS nasIdentifier.")
	        return false
        end
    end
    if (configRow["radiusClient.isEnable_1"] == "1") then
        if (configRow["radiusClient.authsecret2"] == nil or configRow["radiusClient.authsecret2"] == '') then
            printCLIError ("Invalid Backup RADIUS server secret.")
	        return false
        end
        if (configRow["radiusClient.authserver2"] == nil or configRow["radiusClient.authserver2"] == '' or
            configRow["radiusClient.authserver2"] == "0.0.0.0") then
            printCLIError ("Invalid Backup RADIUS server IP.")
	        return false
        end
        if (configRow["radiusClient.nasIdentifier_1"] == nil or configRow["radiusClient.nasIdentifier_1"] == '') then
            printCLIError ("Invalid Backup RADIUS server nasIdentifier.")
	        return false
        end
    end
--    if (tonumber(configRow["radiusClient.authtimeout"]) > 99) then
--        printCLIError ("Invalid Timeout value, enter a value less than 99.")
--	return false
--    end
    if (tonumber(configRow["radiusClient.authtimeout"]) > 99 or tonumber(configRow["radiusClient.authtimeout"]) < 1) then
        printCLIError ("Invalid Timeout value, enter a value between 1 - 99")
	return false
    end
    if (tonumber(configRow["radiusClient.authretries"]) > 99 or tonumber(configRow["radiusClient.authretries"]) < 0) then
        printCLIError ("Invalid Retries value, enter a value between 0 - 99.")
	return false
    end

    local radius = db.getRow("radiusClient", "_ROWID_", "1")
    local radius2 = db.getRow("radiusClient", "_ROWID_", "2")
    if (configRow["radiusClient.isEnable"] == "0") then
        configRow["radiusClient.authsecret"] = radius["radiusClient.authsecret"] or ''
        configRow["radiusClient.authserver"] = radius["radiusClient.authserver"] or ''
        configRow["radiusClient.nasIdentifier"] = radius["radiusClient.nasIdentifier"] or ''
    end
    if (configRow["radiusClient.isEnable_1"] == "0") then
        configRow["radiusClient.authsecret2"] = radius2["radiusClient.authsecret"] or ''
        configRow["radiusClient.authserver2"] = radius2["radiusClient.authserver"] or ''
        configRow["radiusClient.nasIdentifier_1"] = radius2["radiusClient.nasIdentifier"] or ''
    end
    return true
end

--
--
function radiusCfgDel (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""

    errorFlag, statusCode = radius.radius_config(configRow, "-1",
	       "delete")

    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
--
function radiusCfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""

    if (configRow["radiusClient.authsecret"] ~= nil) then
			if (util.isAllMasked (configRow["radiusClient.authsecret"])) then
			configRow["radiusClient.authsecret"] = db.getAttribute ("radiusClient","_ROWID_","1","authsecret") or configRow["radiusClient.authsecret"]
			end
		end
        if (configRow["radiusClient.authsecret2"] ~= nil) then
			if (util.isAllMasked (configRow["radiusClient.authsecret2"])) then
			configRow["radiusClient.authsecret2"] = db.getAttribute ("radiusClient","_ROWID_","2","authsecret") or configRow["radiusClient.authsecret2"]
			end
		end
		
		local tempInputTable = {}
		tempInputTable["radiusClient.authserver"] = configRow["radiusClient.authserver2"]
		configRow["radiusClient.authserver2"] = nil
        tempInputTable["radiusClient.authport"] = "1812"
        configRow["radiusClient.authport"] = "1812"
		tempInputTable["radiusClient.authtimeout"]=configRow["radiusClient.authtimeout"]
		tempInputTable["radiusClient.authretries"]=configRow["radiusClient.authretries"]
        tempInputTable["radiusClient.authsecret"]=configRow["radiusClient.authsecret2"]
        configRow["radiusClient.authsecret2"] = nil
        tempInputTable["radiusClient.isEnable"]=configRow["radiusClient.isEnable_1"]
        configRow["radiusClient.isEnable_1"] = nil
        tempInputTable["radiusClient.nasIdentifier"]=configRow["radiusClient.nasIdentifier_1"]
        configRow["radiusClient.nasIdentifier_1"] = nil
		errorFlag, statusCode = radius.radius_config(configRow, "1", "edit")

		if (errorFlag == "OK") then
			errorFlag, statusCode = radius.radius_config(tempInputTable, "2", "edit")
		end
	
		-- save db if no error
		if (errorFlag == "OK") then db.save() end
	    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	return errorFlag,statusMessage
    -- Check if the row id field exists. If it does not then we are
    -- adding a new row.
--    if (configRow["radiusClient._ROWID_"] == nil) then
--        errorFlag, statusCode = radius.radius_config (configRow, "-1", "add")
	-- Update the RowId
--	configRow["radiusClient._ROWID_"] = 
--	    db.getAttribute("radiusClient", "radiusClient.authserver", 
--	        configRow["radiusClient.authserver"], "_ROWID_")
--    else
--        errorFlag, statusCode = radius.radius_config (configRow,
--		   configRow["radiusClient._ROWID_"], "edit")
--    end
    -- Save db if no error
--    if (errorFlag == "OK") then db.save() end
--    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

--    return errorFlag, statusMessage
end

--
--
function radiusCfgInit (args)
--    local primKey = args[1] -- Profile name

--    local configRow = db.getRow("radiusClient", "radiusClient.authserver", 
--		    primKey)
    local configRow = db.getRow("radiusClient", "_ROWID_", "1")
    local configRow2 = db.getRow("radiusClient", "_ROWID_", "2")
	configRow["radiusClient.isEnable_1"] = configRow2["radiusClient.isEnable"]
	configRow["radiusClient.authsecret2"] = configRow2["radiusClient.authsecret"]
	configRow["radiusClient.nasIdentifier_1"] = configRow2["radiusClient.nasIdentifier"]
	configRow["radiusClient.authserver2"] = configRow2["radiusClient.authserver"]
	if (configRow["radiusClient.authsecret"] ~= nil) then
		configRow["radiusClient.authsecret"] = util.mask (configRow["radiusClient.authsecret"])
	end
    if (configRow["radiusClient.authsecret2"] ~= nil) then
		configRow["radiusClient.authsecret2"] = util.mask (configRow["radiusClient.authsecret2"])
	end
    if configRow == nil then
        configRow = {} --db.getDefaults (true, "radiusClient")
        configRow["radiusClient.authport"] = db.getAttribute ("environment", 
            "name", "RADIUS_DEF_PORT", "value")
        configRow["radiusClient.authtimeout"] = db.getAttribute ("environment", 
            "name", "RADIUS_DEF_TIMEOUT", "value")
        configRow["radiusClient.authretries"] = db.getAttribute ("environment", 
            "name", "RADIUS_DEF_RETRIES", "value")
--        configRow["radiusClient.authserver"] = primKey
    end

    return rowId, configRow
end


--
-- get configuration details for the specified RADIUS server
-- 
local function radiusGetSingle (serverIP)
    local resultTab = {}

    row = db.getRow ("radiusClient", "radiusClient.authserver",
		     serverIP)
    if (row == nil) then --sanity check
       resTab.insertField (resultTab, "", "No such Radius server")
       return resultTab
    end

    resTab.insertField (resultTab, "Auth Server IP Address",
	row["radiusClient.authserver"])
    resTab.insertField (resultTab, "Auth Port", row["radiusClient.authport"])			   
    resTab.insertField (resultTab, "Timeout (in seconds)", 
		       row["radiusClient.authtimeout"])
    resTab.insertField (resultTab, "Retries", row["radiusClient.authretries"])
    resTab.insertField (resultTab, "Secret", row["radiusClient.authsecret"])
	resTab.insertField (resultTab, "NAS Identifier", row["radiusClient.nasIdentifier"])
    return resultTab
end

--
-- get configuration details for all configured RADIUS servers
-- 
local function radiusGetAll ()
    local i = 0
    local resultTab = {}

    radiusTab = db.getTable("radiusClient")
    for k,v in pairs(radiusTab) do
    i = i + 1
    local row = radiusTab[i]
    if (row["radiusClient.isEnable"] == "1") then
	resTab.insertField (resultTab, "Server IP",
	    v["radiusClient.authserver"])
	resTab.insertField (resultTab, "Server Port", v["radiusClient.authport"])			   
	resTab.insertField (resultTab, "Timeout", v["radiusClient.authtimeout"])
	resTab.insertField (resultTab, "Retries", v["radiusClient.authretries"])
	resTab.insertField (resultTab, "NAS Identifier", v["radiusClient.nasIdentifier"])
    end
    end
    return resultTab
end

--
-- main function for radius get
-- 
function radiusGet (args)
    local serverIP = args[1]
    
    if (serverIP == "all") then	
       printLabel ("Configured RADIUS Client")
       resultTab = radiusGetAll()
    else
       printLabel ("RADIUS Configuration")
       resultTab = radiusGetSingle(serverIP)
    end
    resTab.print (resultTab, 0)
end   
