--[[
#### Abhinendra Babu. T
#### TeamF1
#### www.TeamF1.com
#### June 18, 2008
#### File: radvd.lua
#### Description: RADVD Functions
### Revisions:
01a, 24May12, sig, fix for spr 31935
]]--

--************* Requires *************

--************* Initial Code *************

--package bwProfiles
radvd = {}
radvdPrefix = {}

--************* Functions *************
-- RADVD Profiles Configuration
function radvd.config (inputTable, rowid, operation)
    -- validate
--    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
--    end
    return false
end

-- RADVD Profiles Configuration
function radvd.profileConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    valid = radvd.config(inputTable, rowid, operation)
       
    if (rowid == "2") then
        local ConfigPort_Ipv6 = db.getAttribute ("ConfigPort", "_ROWID_", "2", "LogicalIfName")
        if (ConfigPort_Ipv6 ~= "DMZ") then
            db.rollback()
            return "ERROR", "RADVD_DMZ_NOT_CONFIGURED"
        end
    end

    -- return
    if (valid) then
    	db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
    	db.rollback()
	return "ERROR", "RADVD_CONFIG_FAILED"    	
    end
end

-- RADVD Prefix Configuration
function radvdPrefix.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

function num2hex(num)
    local hexstr = '0123456789abcdef'
    local s = ''
    while num > 0 do
        local mod = math.fmod(num, 16)
        s = string.sub(hexstr, mod+1, mod+1) .. s
        num = math.floor(num / 16)
    end
    if s == '' then s = '0' end
    return s
end

-- RADVD Prefix Configuration
function radvdPrefix.profileConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    if (inputTable["radvdLANPrefixPool.radvdPrefixType"] == "1") then
      local  i = tonumber (inputTable["radvdLANPrefixPool.SLAIdentifier"])
      i = num2hex (i)
      inputTable["radvdLANPrefixPool.radvdAdvPrefix"]  = "2002:0:0:" .. i .. "::"
      inputTable["radvdLANPrefixPool.radvdAdvPrefixLength"]  = "64"
    end
    if (operation == "add") then
        if (DBTable == "radvdLANPrefixPool") then
            maxId = db.columnMaxValueWhere("radvdLANPrefixPool", "xmlId", "radvdPrefixType IS NOT NULL")
            if (maxId == "" or maxId == nil) then
                maxId = -1
            end
            inputTable["radvdLANPrefixPool.xmlId"] = tonumber(maxId) + 1
        elseif (DBTable == "radvdDMZPrefixPool") then
            maxId = db.columnMaxValueWhere("radvdDMZPrefixPool", "xmlId", "radvdPrefixType IS NOT NULL")
            if (maxId == "" or maxId == nil) then
                maxId = -1
            end
            inputTable["radvdDMZPrefixPool.xmlId"] = tonumber(maxId) + 1
        end
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    valid = radvdPrefix.config(inputTable, rowid, operation)

    -- return
    if (valid) then
    	db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
    	db.rollback()
	return "ERROR", "RADVD_PREFIX_CONFIG_FAILED"    	
    end
end

-- RADVD Prefix Deletion
function radvdPrefix.deleteProfiles (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["radvdLANPrefixPool._ROWID_"] = v;
	valid = db.delete("radvdLANPrefixPool" , inTable)
	if (not valid) then
	    break
	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "RADVD_PREFIX_DELETE_FAILED"
    end
end

-- RADVD Prefix Deletion
function radvdPrefix.deleteDMZProfiles (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["radvdDMZPrefixPool._ROWID_"] = v;
	valid = db.delete("radvdDMZPrefixPool" , inTable)
	if (not valid) then
	    break
	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "RADVD_PREFIX_DELETE_FAILED"
    end
end
