-- snmpCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 24jan14,nik Added snmpv3 users CLI routines
-- 20jul12,akr Added sysSNMPRemoteSnmpShow and remoteSnmpSetup
-- 05jan10,db  Added snmp Access control routines and
--         modified snmp trap routines
-- 28nov07,gnm written
--
-- Description
-- CLISH SNMP LUA routines

-- load required library
require "teamf1lualib/net-snmp"

--
-- SNMP input validation routine
function snmpSysConfigInputVal (args)
    return TRUE
end

--function for showing remote snmp configuration
function sysSNMPRemoteSnmpShow(args)
    local configRow = db.getRow ("RemoteSnmp","RemoteSnmp._ROWID_", 1)
    if ( configRow == nil ) then
	printCLIError("Error in getting configuration.")
	return false
    end
    printLabel("Remote SNMP Configuration")
    if( configRow["RemoteSnmp.Status"] == "1") then
	print("Remote SNMP :","Enabled")
    elseif( configRow["RemoteSnmp.Status"] == "0") then
	print("Remote SNMP :","Disabled")
    end
    return true
end 

--function for configuration o remote snmp
function remoteSnmpSetup(args)
    local errorFlag = "ERROR" -- default 
    local statusCode = ""
    local statusMessage = ""
    local configRow = db.getRow ("RemoteSnmp","RemoteSnmp._ROWID_", 1)
    if ( configRow == nil ) then
	configRow = {}
    end
    configRow["RemoteSnmp.Status"] = args[1] -- status from user(0/1)
    -- Write configuration changes to the database
    errorFlag, statusCode = snmp.remote_snmp (configRow, "1", "edit")
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    print(statusMessage)
    return errorFlag, statusMessage	
end

--
-- This routine save user password configuration information
function snmpSysConfigSave (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = ""
    local statusMessage = ""

    if (configRow["system.sysContact"] == nil) then
       statusMessage = "Please enter valid system contact value"
       return errorFlag, statusMessage
    elseif (configRow["system.name"] == nil) then	   
       statusMessage = "Please enter valid system name"
       return errorFlag, statusMessage
    elseif (configRow["system.sysLoc"] == nil) then
       statusMessage = "Please enter valid system location"
       return errorFlag, statusMessage
    end
	
    -- Write configuration changes to the database
    errorFlag, statusCode = snmp.system_config (configRow, "1", "edit")
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function snmpSysConfigInit (args)
    local primKey = args[1] -- Agent server IP address in the snmpTrap table

    local configRow = db.getRow ("system","system._ROWID_", 1)
    if configRow == nil then
       configRow = {} -- Create an empty table
    end
           	      
    -- for k, v in pairs(configRow) do print (k, type(v), v) end
    return rowId, configRow
end

--
-- SNMP input validation routine
function snmpTrapConfigInputVal (args)
    return TRUE
end

--
-- This routine save user password configuration information
function snmpTrapConfigDel (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = "SNMP_CONFIG_FAILED"
    local statusMessage = ""
    local rows = {}
    
    table.insert (rows, configRow["snmpTrap._ROWID_"])
    if (rows) then
        errorFlag, statusCode = snmp.snmpDelete(rows)
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
-- This routine save user password configuration information
function snmpTrapConfigSave (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = ""
    local statusMessage = ""

    -- Input validation
    if (configRow["snmpTrap.snmpVersion"] == nil) then
        statusMessage = "Invalid snmp version."
        return errorFlag, statusMessage
    end
    if (configRow["snmpTrap.ipAddr"] == nil) then
        printCLIError("Invalid IP address.")
        return false
    elseif (configRow["snmpTrap.port"] == nil) then
	printCLIError("Invalid port number.")
        return false
    elseif (tonumber(configRow["snmpTrap.port"]) < 1025 and tonumber(configRow["snmpTrap.port"]) ~= 162 and tonumber(configRow["snmpTrap.port"]) ~= 161 ) then
	printCLIError("Invalid port number,Entered port is reserved. Please enter default port - 162,161 or a port greater than 1024.")
        return false
    elseif (configRow["snmpTrap.commName"] == nil) then
        printCLIError("Please enter a valid SNMP community string.")
	    return false
    elseif (configRow["snmpTrap.subnetMask"] == nil) then
        printCLIError("Please enter a valid Subnet mask.")
	    return false
    end

    -- Check if the row id field exists. If it does not then we  adding a
    -- new row.
    if (configRow["snmpTrap._ROWID_"] == nil) then
        errorFlag, statusCode = snmp.snmpTrap_config (configRow, "-1", "add")
	-- Update the RowId
	configRow["snmpTrap._ROWID_"] = db.getAttribute("snmpTrap", 
				      "snmpTrap.ipAddr", configRow["snmpTrap.ipAddr"],
				       "_ROWID_")
    else
        errorFlag, statusCode = snmp.snmpTrap_config (configRow, 
		   configRow["snmpTrap._ROWID_"], "edit")
    end

    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)    

    return errorFlag, statusMessage
end

--
-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function snmpTrapConfigInit (args)
    local primKey = args[1] -- Agent server IP address in the snmpTrap table

    local configRow = db.getRow("snmpTrap", "snmpTrap.ipAddr", primKey)
    if configRow == nil then
       configRow = {} -- Create an empty table
       configRow["snmpTrap.ipAddr"] = primKey
    end
    configRow["snmpTrap.snmpVersion"] = "v1"
    configRow["snmpTrap.subnetMask"] = db.getAttribute("snmpAccessControl","ipAddr",configRow["snmpTrap.ipAddr"],"subnetMask")
           	      
    -- for k, v in pairs(configRow) do print (k, type(v), v) end
    return rowId, configRow
end

--
-- SNMP input validation routine access Control
--function snmpACLConfigInputVal (args)
--    return TRUE
--end

--
-- This routine save user password configuration information
--function snmpACLConfigDel (configRow)
--    local errorFlag = "ERROR" -- default 
--    local statusCode = "SNMP_CONFIG_FAILED"
--    local statusMessage = ""
--    local rows = {}
    
--    table.insert (rows, configRow["snmpAccessControl._ROWID_"])
--    if (rows) then
--        errorFlag, statusCode = snmp.snmpAccessControl_config(rows, "-1", "delete")
--    end
    -- save db if no error
--    if (errorFlag == "OK") then db.save() end
--    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
--    return errorFlag, statusMessage
--end

--
-- This routine save user password configuration information
--function snmpACLConfigSave (configRow)
--    local errorFlag = "ERROR" -- default 
--    local statusCode = ""
--    local statusMessage = ""

    -- Input validation
--    if (configRow["snmpAccessControl.subnetMask"] == nil) then
--        statusMessage = "Invalid subnet mask."
--        return errorFlag, statusMessage
--    elseif (configRow["snmpAccessControl.ipAddr"] == nil) then
--        statusMessage = "Invalid IP address."
--        return errorFlag, statusMessage
--    elseif (configRow["snmpAccessControl.accessType"] == nil) then
--	statusMessage = "Invalid port number."
--        return errorFlag, statusMessage
--    elseif (configRow["snmpAccessControl.commName"] == nil) then
--        statusMessage = "Please enter a valid SNMP community string."
--	return errorFlag, statusMessage
--    end

    -- Check if the row id field exists. If it does not then we  adding a
    -- new row.
--    if (configRow["snmpAccessControl._ROWID_"] == "-1") then
--        errorFlag, statusCode = snmp.snmpAccessControl_config (configRow, "-1", "add")
--    else
--        errorFlag, statusCode = snmp.snmpAccessControl_config (configRow, 
--		   configRow["snmpAccessControl._ROWID_"], "edit")
--    end

--    if (errorFlag == "OK") then db.save() end
--    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)    

--    return errorFlag, statusMessage
--end

--
-- This routine takes in one argument that is the value of the
-- primary key for the object table.
--function snmpACLConfigInit (args)
-- for add
--    local configRow = {}
--    if (args[1] == "-1") then
--        configRow ["snmpAccessControl._ROWID_"] = args[1]
        -- for edit
--    else
--        configRow = db.getRow("snmpAccessControl", "snmpAccessControl._ROWID_", args[1])
--    end
--    return configRow ["snmpAccessControl._ROWID_"], configRow
--end

function sysSNMPShow ()
    local resultTab = {}
    local resultTabACL = {}
       
    if (agentIP == nil) then
        local snmpTab = db.getTable("snmpTrap")
	local idx = 0
	for k,v in pairs(snmpTab) do
	    idx = idx + 1
	    local row = snmpTab[idx]
	    row["snmpTrap.subnetMask"] = db.getAttribute("snmpAccessControl","ipAddr",row["snmpTrap.ipAddr"],"subnetMask")
        resTab.insertField (resultTab, "IP Address", row["snmpTrap.ipAddr"])
        resTab.insertField (resultTab, "Subnet Mask", row["snmpTrap.subnetMask"])
       resTab.insertField (resultTab, "Snmp Version", row["snmpTrap.snmpVersion"])
        resTab.insertField (resultTab, "Port", row["snmpTrap.port"])
        resTab.insertField (resultTab, "Community", row["snmpTrap.commName"])
        end
    else 
	local row = db.getRow ("snmpTrap", "snmpTrap.ipAddr", agentIP)	
    if (row == nil) then return end --sanity check

    row["snmpTrap.subnetMask"] = db.getAttribute("snmpAccessControl","ipAddr",row["snmpTrap.ipAddr"],"subnetMask")
    resTab.insertField (resultTab, "IP Address", row["snmpTrap.ipAddr"])
    resTab.insertField (resultTab, "Subnet Mask", row["snmpTrap.subnetMask"])
     resTab.insertField (resultTab, "Snmp Version", row["snmpTrap.snmpVersion"])
    resTab.insertField (resultTab, "Port", row["snmpTrap.port"])
    resTab.insertField (resultTab, "Community", row["snmpTrap.commName"])
    end
    printLabel ("Trap Agent IP Address")
    resTab.print (resultTab, 0)
end

function sysSNMPConfigShow ()
    local resultTab = {}

    row = db.getRow ("system","system._ROWID_", 1)
    if (row == nil) then
       return nil
    end
    resTab.insertField (resultTab, "SysContact", row["system.sysContact"] or "")
    resTab.insertField (resultTab, "SysLocation", row["system.sysLoc"] or "")
    resTab.insertField (resultTab, "SysName", row["system.name"])
    if (resultTab ~= nil) then
       printLabel ("SNMP System Configuration")
       resTab.print (resultTab, 0)
    end
end

function getSysSnmpTrapEvents (args)
    printLabel ("SNMP Trap Events")
    row = db.getRow("SnmpTrapEvents", "SnmpTrapEvents._ROWID_",1)
--  The SnmpTrapEvents table should have only one entry
     
    if (row["SnmpTrapEvents.wanConnFailEvent"] == "0") then
       print( " Trap WAN Connection Failure Event : No")      
    else 
       print( " Trap WAN Connection Failure Event : Yes")    
    end
    if (row["SnmpTrapEvents.firewallEvent"] == "0") then
       print( " Trap SNMP Firewall events         : No")
    else 
       print( " Trap SNMP Firewall events         : Yes")    
    end
    if (row["SnmpTrapEvents.ipSecVPNEVent"] == "0") then
       print( " Trap SNMP IPSec-VPN events        : No")      
    else 
       print( " Trap SNMP IPSec-VPN events        : Yes")    
    end
    if(row["SnmpTrapEvents.sslVPNEVent"] == "0") then
        print(" Trap SNMP SSL-VPN events          : No")
    else
        print(" Trap SNMP SSL-VPN events          : Yes")
    end
    if(row["SnmpTrapEvents.userLoginEvent"] == "0") then
        print(" Trap SNMP User Login events       : No")
    else
        print(" Trap SNMP User Login events        : Yes")
    end
    if(row["SnmpTrapEvents.userLoginFailEvent"] == "0") then
        print(" Trap SNMP User-Login Failure event: No")
    else
        print(" Trap SNMP User-Login Failure event: Yes")
    end
    if(row["SnmpTrapEvents.wanFailOverEvent"] == "0") then
        print(" Trap SNMP WAN Fail-Over events    : No")
    else
        print(" Trap SNMP WAN Fail-Over events    : Yes")
    end
    if(row["SnmpTrapEvents.configChangeEvent"] == "0") then
        print(" Trap Configuration change events  : No")
    else
        print(" Trap Configuration change events  : Yes")
    end
    
end


function snmpTrapEventInit(args)
local configRow = db.getRow("snmpTrapEvents", "_ROWID_", "1")
return 1,configRow
end


function snmpTrapEventVal(args)
    return TRUE
end

function snmpTrapEventSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
   
    errorFlag, statusCode = snmp.snmpTrapEventsConfig(configRow)

    -- save db if no error
    if (errorFlag == "OK") then	db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function snmpv3UsersConfigInit (args)
    local configRow = db.getRow("snmpv3Users", "snmpv3Users.userName", args[1])
    if (configRow == nil) then
        printCLIError("User doesn't exist.")
	    return -1, {}
    end
           	      
    return configRow["snmpv3Users._ROWID_"], configRow
end

function snmpv3UsersConfigInputVal (configRow)
    if (configRow["snmpv3Users.securityLevel"] == "1" or configRow["snmpv3Users.securityLevel"] == "2") then
        if (configRow["snmpv3Users.authAlgo"] == "MD5" and string.len(configRow["snmpv3Users.authPassword"]) ~= 16) then
            printCLIError("For MD5 Key must be 16 characters long.")
            return false
        end
        if (configRow["snmpv3Users.authAlgo"] == "SHA" and string.len(configRow["snmpv3Users.authPassword"]) ~= 20) then
            printCLIError("For SHA1 Key must be 20 characters long.")
            return false
        end
    end
    if (configRow["snmpv3Users.securityLevel"] == "2") then
        if (configRow["snmpv3Users.privAlgo"] == "DES" and string.len(configRow["snmpv3Users.privPassword"]) ~= 8) then
            printCLIError("For DES Key must be 8 characters long.")
            return false
        end
        if (configRow["snmpv3Users.privAlgo"] == "AES" and string.len(configRow["snmpv3Users.privPassword"]) ~= 16) then
            printCLIError("For AES Key must be 16 characters long.")
            return false
        end
    end
    return true
end

function snmpv3UsersConfigSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
   
    errorFlag, statusCode = snmp.snmpv3Users_config(configRow, configRow["snmpv3Users._ROWID_"], "edit")

    -- save db if no error
    if (errorFlag == "OK") then	db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or ''
    print(statusMessage)
    return errorFlag, statusMessage
end

function sysSNMPv3UsersShow ()
    local resultTab = {}
       
    local snmpv3Tab = db.getTable("snmpv3Users")
	local idx = 0
	for k,v in pairs(snmpv3Tab) do
	    idx = idx + 1
	    local row = snmpv3Tab[idx]
        resTab.insertField (resultTab, "User Name", row["snmpv3Users.userName"])
        resTab.insertField (resultTab, "Access Type", row["snmpv3Users.accessType"])
        local secLevel = ''
        if (row["snmpv3Users.securityLevel"] == "0") then
            secLevel = "NoAuthNoPriv"
        elseif (row["snmpv3Users.securityLevel"] == "1") then
            secLevel = "AuthNoPriv"
        elseif (row["snmpv3Users.securityLevel"] == "2") then
            secLevel = "AuthPriv"
        end
        resTab.insertField (resultTab, "Security Level", secLevel)
    end
    printLabel ("SNMPv3 Users")
    resTab.print (resultTab, 0)
end
