--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: util.lua
#### Description: Utility (common) functions.

#### Revisions:
01a,17Apr08, shv sending enpty string instead of "..." in case of
                empty IP Address joing in joinFields function
01b,23Sep08, shv added new function to get month number from month name
01c,24Apr12, akr added functions: ipv6RoutingTable and ipv4RoutingTable
01d,05mar13, cg  added api's for cms
01e,19sep13, ksr fixed SPR 39999
]]--


--************* Requires *************
require "timeLib"
require "ifDevLib"

--************* Initial Code *************


-- package util
util = {}

-- global variables
_DEBUG_STATUS = false
_DEBUG_OUTPUT = "<br>"
_OUTPUT_STATUS = false
_OUTPUT = "<br>"
_XML_OUTPUT = ""
util.v6ValidationTbl = {}
--************* Functions *************

-- Toggle debugging.
function util.setDebugStatus (onoff)
	_DEBUG_STATUS = onoff
end

-- Get debugging status.
function util.getDebugStatus ()
	return _DEBUG_STATUS
end

-- Append debugging.
function util.appendDebugOut (stuff)
	if _DEBUG_STATUS then
		_DEBUG_OUTPUT = _DEBUG_OUTPUT .. stuff
	end
end

-- Get debugging.
function util.getDebugOut ()
	return _DEBUG_OUTPUT
end

-- Toggle output.
function util.setOutputStatus (onoff)
	_OUTPUT_STATUS = onoff
end

-- Get output status.
function util.getOutputStatus ()
	return _OUTPUT_STATUS
end

-- Append output.
function util.appendOut (stuff)
	if _OUTPUT_STATUS then
		_OUTPUT = _OUTPUT .. stuff
	end
end

-- Get output.
function util.getOut ()
	return _OUTPUT
end

-- Append xml output.
function util.appendXmlOut (stuff)
    _XML_OUTPUT = _XML_OUTPUT .. stuff
end

-- Get xml output.
function util.getXmlOut ()
	return _XML_OUTPUT
end

function util.resetXmlOut ()
	_XML_OUTPUT = ""
end
-- Read file and return as string.
function util.fileToString (filename)
	local filep = io.open(filename, "r")
	return filep:read("*all")
end

-- Check if file exists.
function util.fileExists(filename)
	local filep = io.open(filename)
	if filep then
		io.close(filep)
		return true
	else
		return false
	end
end

-- Convert the table to a string.
function util.tableToString (table)
	if (table == nil) then return "Nil table!" end
	local toReturn = "----- Table: -----<br>"
	for k,v in pairs(table) do
		toReturn = toReturn .. "Key = " .. k .. ", Value = " .. v .. "<br>"
	end
	return toReturn .. "----------------------<br>"
end

-- Convert the table recursively to a string.
function util.tableToStringRec (t, indent)
	if (table == nil) then return "Nil table!" end
	return "----- Recusive Table: -----<br>" .. util.tableToStringRecHelper(t, indent) .. "-------------------------------------<br>"
end

-- Helper - Convert the table recursively to a string.
function util.tableToStringRecHelper (t, indent)
	local toReturn = ""
	local indent = indent or ''
	for key,value in pairs(t) do
    	toReturn = toReturn .. indent .. '[' .. tostring(key) .. ']'
    	if type(value)=="table" then toReturn = toReturn .. ':\n<br>' .. util.tableToStringRecHelper(value, indent .. '&nbsp&nbsp&nbsp')
    	else toReturn = toReturn .. ' = ' .. tostring(value) .. '\n<br>' end
  	end
	return toReturn
end

-- Return table1 - table2 (subtract all from table1 whose
-- values are not in table2).
function util.tableSubtract (table1, table2)
	if (table1 == nil) then return nil end
	if (table2 == nil) then return table1 end

	local result = util.filterTable(table1,
		function (v)
			for key, value in pairs(table2) do
         		if v == value then
         			return false
         		end
      		end
      		return true
      	end
	)
	return result
end

-- Generic table filter function. Filter table with given
-- boolean fucntion.
function util.filterTable (t, f)
	assert (type (t) == "table")
	assert (type (f) == "function")
	local k, v
	local result = {}
	for k, v in ipairs (t) do
    	if f (v) then
			table.insert(result, v)
		end -- if
  	end -- for
  	return result
end -- filtertable

-- Return new table table1 + table2 (add tables).
-- Overlapping keys will only have one entry.
function util.tableAdd (table1, table2)
	local result = {}
	for k, v in pairs (table1) do
		result[k] = v
  	end
  	for k, v in pairs (table2) do
		result[k] = v
  	end
	return result
end

-- Append table2 to table1 (append tables).
-- Overlapping keys will only have one entry.
function util.tableAppend (table1, table2)
	if (table1 == nil or table2 == nil) then return table1 end
  	for k, v in pairs (table2) do
		table1[k] = v
  	end
	return table1
end

-- Split string with divider and return table of parts.
function util.split (str, div)
    if (div=='') then return false end
    local pos,arr = 0,{}
    -- for each divider found
    for st,sp in function() return string.find(str,div,pos,true) end do
	    table.insert(arr,string.sub(str,pos,st-1)) -- Attach chars left of current divider
	    pos = sp + 1 -- Jump past current divider
    end
    table.insert(arr,string.sub(str,pos)) -- Attach chars right of last divider
    return arr
end

-- Join table of parts with divider and return string.
function util.join (intable, div)
    if (not intable) then return "" end
    local str = {}
    -- make string
    for k, v in pairs (intable) do
    	if (k >= 2) then
			str = str .. div .. v
		else
			str = v
		end
  	end
  	return str
end

-- Returns table with entries from given table
-- where key's firstPart matches.
-- Returns nil if none found or error.
function util.tableSplit (table, div, firstPart)
	local splitted = {}
	local toReturn = {}
	for k,v in pairs(table) do
		splitted = util.split(k, div)
		if (#splitted == 2 and splitted[1] == firstPart) then
			toReturn[splitted[1] .. "." .. splitted[2]] = v
		end
	end
	if util.tableSize(toReturn) > 0 then
		return toReturn
	else
		return {}
	end
end

-- Search for given key in table. If matched, returns true.
function util.tableKeyExists (table, key)
	local toReturn = false
	for k,v in pairs(table) do
		if (k == key) then
			return true
		end
	end
	return toReturn
end

-- Search for given value in table. If matched, returns key, else nil.
function util.keyTableValueExists (table, value)
	local toReturn = false
	for k,v in pairs(table) do
		if (v == value) then
			return k
		end
	end
	return nil
end

-- Get size of table.
function util.tableSize (table)
	--if nil
	if table == nil then return nil end
	local i = 0
	for k,v in pairs(table) do
		i = i + 1
	end
	return i
end

-- Iterator that returns pairs in table sorted by keys.
function util.pairsByKeys (t, f)
	local a = {}
    for n in pairs(t) do table.insert(a, n) end
	table.sort(a, f)
	local i = 0      -- iterator variable
	local iter = function ()   -- iterator function
    	i = i + 1
    	if a[i] == nil then return nil
    	else return a[i], t[a[i]]
    	end
    end
    return iter
end

-- Return lua variable with given name in the global environment
-- (does not use lexical scoping).
function util.getLuaVariable (varname)
	local f = loadstring("return " .. varname)
	return f()
end

-- Copy given file to other file.
function util.copyFile (fileHandle, filename)
	local BUFSIZE = 2^13 -- 8 KB
	local chunk = ""
	local fp = io.open(filename, "wb")

	while (true) do
		chunk = fileHandle:read(BUFSIZE)
		if not chunk then break end
		assert(fp:write(chunk))
		assert(fp:flush())
	end
	-- close handles
	fp:close()
	fileHandle:close()
end

-- Get system time in a default format.
function util.time ()
	return os.date("%A, %B %d, %Y, %X (GMT %z)")
end

-- Get system time and date.
-- Returns array of time and date info.
function util.date ()
	local info = {}
	info["month"] = os.date("%m")
	info["date"] = os.date("%d")
	info["year"] = os.date("%Y")
	info["hours"] = os.date("%H")
	info["minutes"] = os.date("%M")
	info["seconds"] = os.date("%S")
	return info
end

-- Get system up time.
function util.uptime ()
    local status,uptime = timeLib.uptime()
    local mins = math.floor(uptime / 60)
	local sec = uptime % 60
    local hours = math.floor (mins / 60)
    mins = mins % 60
	local days = math.floor (hours / 24)
    hours = hours % 24
	uptime = days .. " days, " .. hours .. " hours, " .. mins .. " minutes, " .. sec .. " seconds"
	return uptime
end

-- Returns an array of bits (0 or 1 integer) based on integer (base 10) value.
-- First entry in array is the least significant bit (LSB), last is MSB.
-- Ex: integer 15 would return {1,1,1,1}, integer 10 would return {0,1,0,1}
function util.integerToBits (number)
	local toReturn = {}
	local remainder = 0
	local quotient = -1

	while (quotient ~= 0) do
		quotient = math.floor(number / 2)
		remainder = number % 2
		number = quotient
		table.insert(toReturn, remainder)
	end

	-- return
	return toReturn
end

-- Returns array of values in given input table that are keyed by <tablename><div><fieldname><i>
-- where i ranges from 1 to length using the given divider.
function util.getPartialArray (fulltable, tablename, fieldname, div, length)
	local toReturn = {}
	local i = 1

	while (i <= length) do
		table.insert(toReturn, fulltable[tablename .. div .. fieldname .. i])
		i = i + 1
	end
	return toReturn
end

-- Join the values in given input table that are keyed by <tablename><div><fieldname><i>
-- where i ranges from 1 to length using the given divider.
function util.joinFields (table, tablename, fieldname, div, length)
	local toReturn = ""
	local i = 1

	while (i <= length) do
	    if(table[tablename .. div .. fieldname .. i] ~= nil) then
		    toReturn = toReturn .. table[tablename .. div .. fieldname .. i] .. div
		    i = i + 1
		else
		    return nil
		end
	end
	toReturn = toReturn:sub(1, toReturn:len()-div:len()) --shave last dot

    if (toReturn == "...") then
        toReturn = ""
    end

	return toReturn
end

-- Join the bit values in given input table that are keyed by <tablename><div><fieldname><i>
-- where i ranges from 1 to length using the given divider.
-- -reverse- dictates that first index should be LSB, not MSB.
function util.joinBitFields (table, tablename, fieldname, div, length, reverse)
	local binaryArray = util.getPartialArray(table, tablename, fieldname, div, length)
	local binaryString = ""

	if (reverse) then
		local i = length
		while (i >= 1) do
			binaryString = binaryString .. binaryArray[i]
			i = i - 1
		end
	else
		local i = 1
		while (i <= length) do
			binaryString = binaryString .. binaryArray[i]
			i = i + 1
		end
	end
	return tonumber(binaryString, 2) --to binary
end

--
--
-- Convert ASCII string to integer
-- ToDo: Add check for largest and smallest int
function util.atoi (str)
    local len = string.len (str)
    local intVal = 0
    local idx = 0
    local startIdx = 1

    if (string.byte (str, 1) == 45) then
        startIdx = 2
        base = math.pow (10, len-2)
    else
        startIdx = 1
        base = math.pow (10, len-1)
    end

    for idx = startIdx,len do
        local byteVal = string.byte (str, idx)
        if byteVal >= 48 and byteVal <= 58 then
            byteVal = byteVal - string.byte ("0", 1)
            intVal = intVal + base * byteVal
            base = base/10
        else
            return -1, 0
        end
    end
    if startIdx == 2 then intVal = intVal * -1 end
    return 0, intVal
end

-- Mask Password/Secret
function util.mask (str)
    local len = string.len (str)
    local startIdx = 1
    local maskString = ""

    for idx = startIdx,len do
        maskString = maskString .. "*"
    end

    return maskString
end

-- Check for all '*' in  Password/Secret fields
function util.isAllMasked (str)
    if (str == "") then
        return false
    end
    local maskedStr = util.mask (str)
    local allMasked = true

    if (str ~= maskedStr) then
        allMasked = false
    end

    return allMasked
end

-- Mask Password/Secret filling with'@'
function util.mask_new (str)
    local len = string.len (str)
    local startIdx = 1
    local maskString = ""

    for idx = startIdx,len do
        maskString = maskString .. "@"
    end

    return maskString
end

-- Check for all '@' in  Password/Secret fields
function util.isAllMasked_new (str)
    if (str == "") then
        return false
    end
    local maskedStr = util.mask_new (str)
    local allMasked = true

    if (str ~= maskedStr) then
        allMasked = false
    end

    return allMasked
end

function util.splitDateTime(str)
    local dateTime = {}
    local strSplit  = util.split(str, "@")
    local date = util.split(strSplit[1], "/")
    dateTime[1] = date[2]
    dateTime[2] = date[3]
    dateTime[3] = date[1]
    local time = util.split(strSplit[2], ":")
    dateTime[4] = time[1]
    dateTime[5] = time[2]
    dateTime[6] = time[3]
    return dateTime
end

function util.getMonthNum (month)
    if (month == nil) then
        return 0
    elseif (month == "Jan") then
        return 1
    elseif (month == "Feb") then
        return 2
    elseif (month == "Mar") then
        return 3
    elseif (month == "Apr") then
        return 4
    elseif (month == "May") then
        return 5
    elseif (month == "Jun") then
        return 6
    elseif (month == "Jul") then
        return 7
    elseif (month == "Aug") then
        return 8
    elseif (month == "Sep") then
        return 9
    elseif (month == "Oct") then
        return 10
    elseif (month == "Nov") then
        return 11
    elseif (month == "Dec") then
        return 12
    else
        return 0
    end
end

function util.getDisplayRecords (tableName, fullName, customQuery)
    local outTable = nil

    local rows = db.getTable (tableName, fullName, customQuery)
    if (rows ~= nil and #rows > 0) then
        TOTAL_REC_COUNT = #rows
        -- If number of rows <= Records Display Limit
        if (#rows <= RECORDS_LIMIT) then
            outTable = rows
            FROM_RECORD_NUM = 1
            TO_RECORD_NUM = #rows
        else
            local rowidCheck = true
            local recIndex = 0
            for k,v in pairs (rows) do
                if (rowidCheck) then
                    FROM_RECORD_NUM = FROM_RECORD_NUM + 1
                    local rowid = v[tableName .. "._ROWID_"] or v["_ROWID_"]
                    if (FROM_RECORD_NUM == 1) then FIRST_REC_ID = rowid end
                    if (tonumber (rowid) >= RECORD_ID) then
                        rowidCheck = false
                        outTable = {}
                        -- Set previoud records starting rowid
                        if (FROM_RECORD_NUM > RECORDS_LIMIT) then
                            PREVIOUS_REC = rows[FROM_RECORD_NUM - RECORDS_LIMIT][tableName .. "._ROWID_"] or rows[FROM_RECORD_NUM - RECORDS_LIMIT]["_ROWID_"]
                        else
                            PREVIOUS_REC = rows[1][tableName .. "._ROWID_"] or rows[1]["_ROWID_"]
                        end
                    end
                end

                if (not rowidCheck) then
                    recIndex = recIndex + 1
                    if (recIndex > RECORDS_LIMIT) then
                        -- Set Next record rowid
                        NEXT_REC = v[tableName .. "._ROWID_"] or v["_ROWID_"]

                        -- Set Last record rowid
                        if (#rows > RECORDS_LIMIT) then
                            LAST_REC = rows[#rows - RECORDS_LIMIT + 1][tableName .. "._ROWID_"] or rows[#rows - RECORDS_LIMIT + 1]["_ROWID_"]
                        end
                        break
                    end
                    outTable[recIndex] = v
                    TO_RECORD_NUM = FROM_RECORD_NUM + recIndex - 1
                end
            end
        end
    end

    return outTable
end

-- This function will replace & with &amp; less than (<) symbol with html entity &lt; 
-- and greater than symbol (>) with html entity &gt;

function util.xssEncoding(cgiTable)
    local inputTable = {}
    for k, v in pairs (cgiTable) do
        v = string.gsub(v, '&', "&amp;")
        v = string.gsub(v, '<', "&lt;")
        v = string.gsub(v, '>', "&gt;")   
        inputTable[k] = v
    end 
    return inputTable
end

--
--This function will return ipv4 routing table in lua table format
--
function util.ipv4RoutingTable(arg)
		local command = [[ route -n |awk 'BEGIN{ORS=",";}{if(NR>1){for(i=1;i<=NF;i++){print $i};}}' >/var/route.txt]]
		os.execute(command)
	    local f = assert(io.open("/var/route.txt","r"))
	    local line=''
        line = f:read("*line") or ''
   		assert(f:close())
   		local luaTBL= util.split(line,',') or {}
   		return luaTBL or {}
end
--
--This function will return ipv6 routing table in lua table format
--
function util.ipv6RoutingTable(arg)
		local command = [[ /pfrm2.0/bin/ip -6 route|awk 'BEGIN{OFS=",";ORS=","}{nextHop="::";metric="";interface="";for(i=1;i<=NF;++i){destinaton=$1;if($i == "via"){ nextHop=$(i+1)};if($i=="dev"){interface=$(i+1)};if($i=="metric"){metric=$(i+1)};}print $1,nextHop,metric,interface}' >/var/route6.txt]]
		os.execute(command)
	    local f = assert(io.open("/var/route6.txt","r"))
	    local line=''
        line = f:read("*line") or ''
   		assert(f:close())
   		local luaTBL = util.split(line,',') or {}
   		return luaTBL or {}
end

--
-- This function truncates the name to avoid display issues
-- 
function compressDisplayNames(field,len)
        local displayName = ''
        if(field == nil or filed == '') then
        return '','';
        end
        local fieldLen = #field
        if(fieldLen > len) then
           displayName = field
           field = string.sub(field,1,len) .. "..."
        end
        return field,displayName
end

function util.getCompName (t)
    local compName 
    for word in string.gmatch(t, "([^:]*)") do 
        if (word~="") then 
            compName = word 
        end 
    end 
    return compName
end

-- This function does the ipv6 address validation 
function util.ipv6AddressValidation(ipv6Address, addressType)
	if (ipv6Address == nil or ipv6Address == "" ) then 
		if(addressType ~= nil and addressType ~= "" ) then
			statusMessage = "Nil value instead of a valid IPv6 address for " .. addressType .. "."
		else
			statusMessage = "Nil value instead of a valid IPv6 address " 
		end
		table.insert(util.v6ValidationTbl, statusMessage)
		return
	end
	if ( ifDevLib.ifDevTypeOfIPv6Address(ipv6Address) == nil) then
		if(addressType ~= nil and addressType ~= "" ) then
			statusMessage = ipv6Address .. " is not a valid " .. addressType .. " IPv6 address."
		else
			statusMessage = ipv6Address .. " is not a valid IPv6 address."
		end
		table.insert(util.v6ValidationTbl, statusMessage)
		return
	end
end
